package com.pj.ht_import_declare_people;

import java.util.List;

import com.pj.api.dto.ImportDeclarePeopleDto;
import com.pj.utils.so.SoMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.project4sp.SP;

import com.pj.current.satoken.StpUserUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;


/**
 * Controller: ht_import_declare_people -- 进口申报单边民确认表
 * @author yzs 
 */
@RestController
@RequestMapping("/HtImportDeclarePeople/")
public class HtImportDeclarePeopleController {

	/** 底层 Service 对象 */
	@Autowired
	HtImportDeclarePeopleService htImportDeclarePeopleService;

	/** 增 */  
	@RequestMapping("add")
	@SaCheckPermission(HtImportDeclarePeople.PERMISSION_CODE_ADD)
	public AjaxJson add(HtImportDeclarePeople h){
		htImportDeclarePeopleService.add(h);
		h = htImportDeclarePeopleService.getById(SP.publicMapper.getPrimarykey());
		return AjaxJson.getSuccessData(h);
	}

	/** 删 */  
	@RequestMapping("delete")
	@SaCheckPermission(HtImportDeclarePeople.PERMISSION_CODE_DEL)
	public AjaxJson delete(Long id){
		 htImportDeclarePeopleService.delete(id);
		return AjaxJson.getSuccess();
	}
	
	/** 删 - 根据id列表 */  
	@RequestMapping("deleteByIds")
	@SaCheckPermission(HtImportDeclarePeople.PERMISSION_CODE_DEL)
	public AjaxJson deleteByIds(){
		List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class); 
		int line = SP.publicMapper.deleteByIds(HtImportDeclarePeople.TABLE_NAME, ids);
		return AjaxJson.getByLine(line);
	}
	
	/** 改 */  
	@RequestMapping("update")
	@SaCheckPermission(HtImportDeclarePeople.PERMISSION_CODE_EDIT)
	public AjaxJson update(HtImportDeclarePeople h){
		htImportDeclarePeopleService.update(h);
		return AjaxJson.getSuccess();
	}

	/** 查 - 根据id */  
	@RequestMapping("getById")
		@SaCheckPermission(HtImportDeclarePeople.PERMISSION_CODE)
	public AjaxJson getById(Integer id){
		HtImportDeclarePeople h = htImportDeclarePeopleService.getById(id);
		return AjaxJson.getSuccessData(h);
	}

	/** 查集合 - 根据条件（参数为空时代表忽略指定条件） */  
	@RequestMapping("getList")
		@SaCheckPermission(HtImportDeclarePeople.PERMISSION_CODE)
	public AjaxJson getList() { 
		SoMap so = SoMap.getRequestSoMap();
		List<HtImportDeclarePeople> list = htImportDeclarePeopleService.getList(so.startPage());
		return AjaxJson.getPageData(so.getDataCount(), list);
	}


	/* ---           正常业务方法 👆👆  rpc远程调用👇👇                              ---*/


	/** 远程调用: 根据便民身份证号 和 进出口结算单号查询[进口申报单边民确认] 将该对象返还航通 */
	@RequestMapping("rpc/getByIdNoAndPreleportNo")
	public ImportDeclarePeopleDto getByIdNoAndPreleportNo(@RequestParam("idNo")String idNo, @RequestParam("preleportNo")String preleportNo){
		ImportDeclarePeopleDto byIdNoAndPreleportNo = htImportDeclarePeopleService.getByIdNoAndPreleportNo(idNo, preleportNo);
		return byIdNoAndPreleportNo;
	}
	
	

}
