package com.pj.ht_trade_settlement;

import java.io.Serializable;
import java.util.Date;

import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.EqualsAndHashCode;

import lombok.Data;
import lombok.experimental.Accessors;

/**
 * Model: ht_trade_settlement -- 航通交易结算表
 * @author yzs 
 */
@Data
@Accessors(chain = true)
@TableName(HtTradeSettlement.TABLE_NAME)
@EqualsAndHashCode(callSuper = false)
public class HtTradeSettlement extends Model<HtTradeSettlement> implements Serializable {

	// ---------- 模块常量 ----------
	/**
	 * 序列化版本id 
	 */
	private static final long serialVersionUID = 1L;	
	/**
	 * 此模块对应的表名 
	 */
	public static final String TABLE_NAME = "ht_trade_settlement";	
	/**
	 * 此模块对应的权限码 
	 */
	public static final String PERMISSION_CODE = "ht-trade-settlement";
	public static final String PERMISSION_CODE_ADD = "ht-trade-settlement-add";
	public static final String PERMISSION_CODE_EDIT = "ht-trade-settlement-edit";
	public static final String PERMISSION_CODE_DEL = "ht-trade-settlement-del";




	// ---------- 表中字段 ----------
	/**
	 * 预申报编号 
	 */
	@TableId(type = IdType.AUTO)
	private Long id;

	/**
	 * 航通主键
	 */
	private String platSeqNo;	

	/**
	 * 申报单编号 
	 */
	private String declareId;	

	/**
	 * 1-边民，2-商铺业主或其代理人，3-合作社代表，4-互助组代表 
	 */
	private String declareType;	

	/**
	 * 申报人姓名 
	 */
	private String declareName;	

	/**
	 * 身份证号码 
	 */
	private String idno;	

	/**
	 * 商铺编码 
	 */
	private String shopNo;	

	/**
	 * 商铺名称 
	 */
	private String shopName;	

	/**
	 * 申报人类别为3、4 时互助组/合作社备案编号 
	 */
	private String orgPutrecNo;	

	/**
	 * 申报人类别为3、4 时互助组/合租社名称 
	 */
	private String orgName;	

	/**
	 * 申报人类别为4 时边民合作社社会统一信用代码 
	 */
	private String orgScc;	

	/**
	 *  运输工具名称 
	 */
	private String veNo;	

	/**
	 * 业务模式, 进出口标志为进口时，01 区内交易、02 落地加工、03 原装提离。进出口标 志 为 出 口时，04：直接出口，05：出口销售。 
	 */
	private String btrnbBizModecd;	

	/**
	 * 商品流向, 进出口标志为进口时，必填，01-二次销售；02-生活自用；03-落地加工； 
	 */
	@JsonProperty("gFlow")
	private String gFlow;	

	/**
	 * 进出口标志：I进口 E 出口 
	 */
	@JsonProperty("iEFlag")
	private String iEFlag;	

	/**
	 * 监管场所编号 
	 */
	private String fieldCode;	

	/**
	 * 监管场所名称 
	 */
	private String fieldName;	

	/**
	 * 申报口岸代码 
	 */
	private String customsCode;	

	/**
	 * 1-是、0-否 
	 */
	private String carPool;	

	/**
	 * 总金额 
	 */
	private Double sumAmt;	

	/**
	 * 总毛重 
	 */
	private Double sumGrossWt;	

	/**
	 * 总净重 
	 */
	private Double sumNetWt;	

	/**
	 * 运输方式 
	 */
	private String trafMode;	

	/**
	 * 币制 
	 */
	private String currCode;	

	/**
	 * 件数，正整数 
	 */
	private Long packNo;	

	/**
	 * 申 报 时 间 ：yyyy-MM-dd HH:mm:ss 
	 */
	private String declTime;	

	/**
	 * 提运单号 
	 */
	private String billNo;	

	/**
	 * 运输工具航次(班)号 
	 */
	private String voyageNo;	

	/**
	 * 商品名称，多个用英文逗号分割 
	 */
	private String goodsNames;	

	/**
	 * 购买边民列表[     { "platSeqNo": "进出境申报单预申报编号",        "borderName": "边民姓名", "idno": "身份证号",        "borderPutrecNo": "边民备案编号",        "settleBillNo": "结算单号", "buyAmount": "申报金额", "netWt": "净重", "grossWt": "毛重",        "buyQty": "申报数量", "buyQty1": "第一数量",        "buyQty2": "第二数量", "preIeportNo": "进出口预申报单号"     }] 
	 */
	private String bordersList;	

	/**
	 * 边民购买商品列表[     { "platSeqNo": "进出境申报单预申报编号",        "borderName": "边民姓名",        "idno": "身份证号",           "borderPutrecNo": "边民备案编号",        "buyAmount": "申报金额",        "netWt": "净重",          "grossWt": "毛重",        "buyQty": "申报数量",        "buyQty1": "第一数量",        "buyQty2": "第二数量",        "preIeportNo": "进出口预申报单号",        "codeTs": "商品编码",        "goodsName": "商品名称"      }] 
	 */
	private String borderBuyGoodsList;	

	/**
	 * 商铺卖货物列表[       { "platSeqNo": "进出境申报单预申报编号",           "gNo": "预申报编号", "codeTs": "商品编码",           "gName": "商品名称", "gModel": "规格型号",           "storageEnvm": "储存条件：常温 1：冷藏，2：冷冻：3",            "useTo": "用途：01 食用、02 工业用、03 药用、04 种用、05 饲用、06 其他",           "grossWt": "毛重",            "netWt": "净重",            "originCountry": "产销国",            "gQty": "货物数量",            "qty1": "第一数量",            "qty2": "第二数量",            "gUnit": "计量单位",            "unit1": "第一计量单位",            "unit2": "第二计量单位",            "declPrice": "单价",            "declTotal": "总价"       }] 
	 */
	private String sellGoodsList;	


	/**
	 * 完成情况
	 */
	private Integer finishStatus;

	/**
	 * 创建时间
	 */
	private Date createTime;




}
