package com.pj.ht_trade_settlement;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.pj.api.dto.TradeSettlementDto;
import com.pj.project4sp.SP;
import com.pj.utils.sg.AjaxJson;
import com.pj.utils.so.SoMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


/**
 * Controller: ht_trade_settlement -- 航通交易结算表
 * @author yzs 
 */
@RestController
@RequestMapping("/HtTradeSettlement/")
public class HtTradeSettlementController {

	/** 底层 Service 对象 */
	@Autowired
	HtTradeSettlementService htTradeSettlementService;

	/** 增 */  
	@RequestMapping("add")
	@SaCheckPermission(HtTradeSettlement.PERMISSION_CODE_ADD)
	public AjaxJson add(HtTradeSettlement h){
		htTradeSettlementService.add(h);
		h = htTradeSettlementService.getById(SP.publicMapper.getPrimarykey());
		return AjaxJson.getSuccessData(h);
	}

	/** 删 */  
	@RequestMapping("delete")
	@SaCheckPermission(HtTradeSettlement.PERMISSION_CODE_DEL)
	public AjaxJson delete(Long id){
		 htTradeSettlementService.delete(id);
		return AjaxJson.getSuccess();
	}
	
	/** 删 - 根据id列表 */  
	@RequestMapping("deleteByIds")
	@SaCheckPermission(HtTradeSettlement.PERMISSION_CODE_DEL)
	public AjaxJson deleteByIds(){
		List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class); 
		int line = SP.publicMapper.deleteByIds(HtTradeSettlement.TABLE_NAME, ids);
		return AjaxJson.getByLine(line);
	}
	
	/** 改 */  
	@RequestMapping("update")
	@SaCheckPermission(HtTradeSettlement.PERMISSION_CODE_EDIT)
	public AjaxJson update(HtTradeSettlement h){
		htTradeSettlementService.update(h);
		return AjaxJson.getSuccess();
	}

	/** 查 - 根据id */  
	@RequestMapping("getById")
		@SaCheckPermission(HtTradeSettlement.PERMISSION_CODE)
	public AjaxJson getById(String id){
		HtTradeSettlement h = htTradeSettlementService.getById(id);
		return AjaxJson.getSuccessData(h);
	}

	/** 查集合 - 根据条件（参数为空时代表忽略指定条件） */  
	@RequestMapping("getList")
		@SaCheckPermission(HtTradeSettlement.PERMISSION_CODE)
	public AjaxJson getList() { 
		SoMap so = SoMap.getRequestSoMap();
		List<HtTradeSettlement> list = htTradeSettlementService.getList(so.startPage());
		return AjaxJson.getPageData(so.getDataCount(), list);
	}

	/*-----           正常业务逻辑👆👆    rpc远程调用👇👇 -----*/

	/** 远程调用: 添加大订单时同事生成小定单 */
	@RequestMapping("rpc/tradeSettlementDto")
	public boolean tradeSettlementDto(@RequestBody TradeSettlementDto tradeSettlementDto){
		boolean b = htTradeSettlementService.tradeSettlementDto(tradeSettlementDto);
		return b;
	}

	
	
	

}
