package com.pj.ht_trade_settlement;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pj.api.dto.BorderBuyGoodsListDto;
import com.pj.api.dto.BordersListDto;
import com.pj.api.dto.TradeSettlementDto;
import com.pj.current.config.SystemObject;
import com.pj.enummj.*;
import com.pj.tb_goods.TbGoods;
import com.pj.tb_goods.TbGoodsService;
import com.pj.tb_order.TbOrder;
import com.pj.tb_order.TbOrderMapper;
import com.pj.tb_people.TbPeople;
import com.pj.tb_people.TbPeopleMapper;
import com.pj.tb_people.TbPeopleService;
import com.pj.tb_shop.TbShop;
import com.pj.tb_shop.TbShopService;
import com.pj.tb_trade_area.TbTradeArea;
import com.pj.tb_trade_area.TbTradeAreaService;
import com.pj.utils.so.SoMap;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Service: ht_trade_settlement -- 航通交易结算表
 *
 * @author yzs
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class HtTradeSettlementService extends ServiceImpl<HtTradeSettlementMapper, HtTradeSettlement> implements IService<HtTradeSettlement> {

    /**
     * 底层 Mapper 对象
     */
    @Autowired
    HtTradeSettlementMapper htTradeSettlementMapper;
    @Autowired
    private TbPeopleService tbPeopleService;
    @Autowired
    private TbOrderMapper tbOrderMapper;

    @Resource
    private TbTradeAreaService tbTradeAreaService;
    @Resource
    private TbGoodsService tbGoodsService;
    @Resource
    private TbShopService tbShopService;

    /**
     * 增
     */
    void add(HtTradeSettlement h) {
        save(h);
    }

    /**
     * 删
     */
    void delete(Long id) {
        removeById(id);
    }

    /**
     * 改
     */
    void update(HtTradeSettlement h) {
        updateById(h);

    }

    /**
     * 查
     */
    HtTradeSettlement getById(String platSeqNo) {
        return super.getById(platSeqNo);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    List<HtTradeSettlement> getList(SoMap so) {
        return htTradeSettlementMapper.getList(so);
    }


    /**
     * 远程调用: 添加大订单时同时生成边民订单
     */
    public boolean tradeSettlementDto(TradeSettlementDto tradeSettlementDto) {
        HtTradeSettlement htTradeSettlement = new HtTradeSettlement();
        BeanUtils.copyProperties(tradeSettlementDto, htTradeSettlement);
        int insert = htTradeSettlementMapper.insert(htTradeSettlement);
        //获取边民订单集合
        List<BordersListDto> bordersListDtoList = tradeSettlementDto.getBordersListDtoList();

        //获取订单商品列表集合
		List<BorderBuyGoodsListDto> borderBuyGoodsListDtoList = tradeSettlementDto.getBorderBuyGoodsListDtoList();

        TbTradeArea tbTradeArea = tbTradeAreaService.updateHandler(tradeSettlementDto.getFieldCode(), tradeSettlementDto.getFieldName());
        TbShop tbShop = tbShopService.findByCode(tradeSettlementDto.getShopNo());
       Date applyTime= DateUtil.parseDateTime( tradeSettlementDto.getDeclTime());
		bordersListDtoList.forEach(item -> {
			//开始创建边民订单
			TbOrder tbOrder = new TbOrder();
			//航通字段
			BeanUtils.copyProperties(item, tbOrder);

			tbOrder.setCreateTime(new Date()).setPreNo(item.getPreNo())
					.setTradeNo(SystemObject.getREFcode("YJSCDD", new Date()))
					.setTradeAreaId(tbTradeArea.getId()).setTradeAreaName(tbTradeArea.getName());
			tbOrder.setTotalWeight(item.getGrossWt()).setTotalPrice(item.getBuyAmount());

			tbOrder.setApply(ApplyStatus.APPLY_STATUS_ONE.getCode()).setApplyTime(applyTime).setVeNo(tradeSettlementDto.getVeNo());
			tbOrder.setDeleteStatus(DeleteStatus.DELETE_STATUS_ON.getCode());
			tbOrder.setResaleStatus(ResaleStatus.RESALE_STATUS_ZERO.getCode());
			tbOrder.setFinishStatus(OrderStatus.ORDER_STATUS_ZERO.getCode());

			//根据边民身份证号获取边民的订单商品列表(目前逻辑为一个边民对应一个商品)
			List<BorderBuyGoodsListDto> collect = borderBuyGoodsListDtoList.stream().filter(borderBuyGoods -> item.getIdno().equals(borderBuyGoods.getIdno())).collect(Collectors.toList());
			if (collect.size() > 0) {
				//根据商品编号查询商品表
				TbGoods tbGoods = tbGoodsService.findByCode(collect.get(0).getCodeTs());
				if(null != tbGoods) {
					tbOrder.setGoodsId(tbGoods.getId());
					tbOrder.setGoodsNames(tbGoods.getName());
					tbOrder.setGoodsUnit(tbGoods.getUnit());
				}
			}
			//根据身份证查询边民
			TbPeople tbPeople = tbPeopleService.findByIdCarNo(item.getIdno());
			if (tbPeople != null) {
				tbOrder.setBuyUserId(tbPeople.getId());
				tbOrder.setBuyUserName(tbPeople.getName());
				tbOrder.setBuyUserType(String.valueOf(tbPeople.getRole()));
				tbOrder.setGroupId(tbPeople.getGroupId());
			}
			//商品店铺信息
			if (tbShop != null)
				tbOrder.setEnterpriseId(tbShop.getEnterpriseId()).setEnterpriseName(tbShop.getEnterpriseName());

			tbOrderMapper.insert(tbOrder);
		});

        return insert == 1;
    }
}
