package com.pj.tb_cooperative;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.pj.api.dto.HtCooperativeDto;
import com.pj.project4sp.SP;
import com.pj.utils.sg.AjaxJson;
import com.pj.utils.so.SoMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


/**
 * Controller: tb_cooperative -- 合作社
 * @author lsw
 */
@RestController
@RequestMapping("/TbCooperative/")
public class TbCooperativeController {

	/** 底层 Service 对象 */
	@Autowired
    TbCooperativeService tbCooperativeService;

	/** 增 */
	@RequestMapping("add")
	@SaCheckPermission(TbCooperative.PERMISSION_CODE_ADD)
	public AjaxJson add(TbCooperative t){
		tbCooperativeService.add(t);
		t = tbCooperativeService.getById(SP.publicMapper.getPrimarykey());
		return AjaxJson.getSuccessData(t);
	}

	/** 删 */
	@RequestMapping("delete")
	@SaCheckPermission(TbCooperative.PERMISSION_CODE_DEL)
	public AjaxJson delete(Long id){
		 tbCooperativeService.delete(id);
		return AjaxJson.getSuccess();
	}

	/** 删 - 根据id列表 */
	@RequestMapping("deleteByIds")
	@SaCheckPermission(TbCooperative.PERMISSION_CODE_DEL)
	public AjaxJson deleteByIds(){
		List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class);
		int line = SP.publicMapper.deleteByIds(TbCooperative.TABLE_NAME, ids);
		return AjaxJson.getByLine(line);
	}

	/** 改 */
	@RequestMapping("update")
	@SaCheckPermission(TbCooperative.PERMISSION_CODE_EDIT)
	public AjaxJson update(TbCooperative t){
		tbCooperativeService.update(t);
		return AjaxJson.getSuccess();
	}

	/** 查 - 根据id */
	@RequestMapping("getById")
		@SaCheckPermission(TbCooperative.PERMISSION_CODE)
	public AjaxJson getById(Long id){
		TbCooperative t = tbCooperativeService.getById(id);
		return AjaxJson.getSuccessData(t);
	}

	/** 查集合 - 根据条件（参数为空时代表忽略指定条件） */
	@RequestMapping("getList")
		@SaCheckPermission(TbCooperative.PERMISSION_CODE)
	public AjaxJson getList() {
		SoMap so = SoMap.getRequestSoMap();
		List<TbCooperative> list = tbCooperativeService.getList(so.startPage());
		return AjaxJson.getPageData(so.getDataCount(), list);
	}

	/** 改 - 删除状态(0=禁用,1=启用) */
	@RequestMapping("updateDeleteStatus")
	@SaCheckPermission(TbCooperative.PERMISSION_CODE_EDIT)
	public AjaxJson updateDeleteStatus(Long id, Integer value){
		int line = SP.publicMapper.updateColumnById(TbCooperative.TABLE_NAME, "delete_status", value, id);
		return AjaxJson.getByLine(line);
	}


	/*------            正常业务👆👆    rpc远程调用👇👇                         -----------*/


	/** 远程调用: 对合作社信息进行事务处理 */
	@RequestMapping("rpc/cooperativeDto")
	public boolean cooperativeDto(@RequestBody HtCooperativeDto cooperativeDto){
		boolean b = tbCooperativeService.cooperativeDto(cooperativeDto);
		return b;
	}
}
