package com.pj.tb_port_news;

import java.util.List;
import com.pj.utils.so.SoMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.project4sp.SP;

import com.pj.current.satoken.StpUserUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;


/**
 * Controller: tb_port_news -- 口岸咨询
 * @author loovi
 */
@RestController
@RequestMapping("/TbPortNews/")
public class TbPortNewsController {

	/** 底层 Service 对象 */
	@Autowired
	TbPortNewsService tbPortNewsService;

	/** 增 */  
	@RequestMapping("add")
	@SaCheckPermission(TbPortNews.PERMISSION_CODE_ADD)
	public AjaxJson add(TbPortNews t){
		tbPortNewsService.add(t);
		t = tbPortNewsService.getById(SP.publicMapper.getPrimarykey());
		return AjaxJson.getSuccessData(t);
	}

	/** 删 */  
	@RequestMapping("delete")
	@SaCheckPermission(TbPortNews.PERMISSION_CODE_DEL)
	public AjaxJson delete(Long id){
		 tbPortNewsService.delete(id);
		return AjaxJson.getSuccess();
	}
	
	/** 删 - 根据id列表 */  
	@RequestMapping("deleteByIds")
	@SaCheckPermission(TbPortNews.PERMISSION_CODE_DEL)
	public AjaxJson deleteByIds(){
		List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class); 
		int line = SP.publicMapper.deleteByIds(TbPortNews.TABLE_NAME, ids);
		return AjaxJson.getByLine(line);
	}
	
	/** 改 */  
	@RequestMapping("update")
	@SaCheckPermission(TbPortNews.PERMISSION_CODE_EDIT)
	public AjaxJson update(TbPortNews t){
		tbPortNewsService.update(t);
		return AjaxJson.getSuccess();
	}

	/** 查 - 根据id */  
	@RequestMapping("getById")
		@SaCheckPermission(TbPortNews.PERMISSION_CODE)
	public AjaxJson getById(Long id){
		TbPortNews t = tbPortNewsService.getById(id);
		return AjaxJson.getSuccessData(t);
	}

	/** 查集合 - 根据条件（参数为空时代表忽略指定条件） */  
	@RequestMapping("getList")
		@SaCheckPermission(TbPortNews.PERMISSION_CODE)
	public AjaxJson getList() { 
		SoMap so = SoMap.getRequestSoMap();
		List<TbPortNews> list = tbPortNewsService.getList(so.startPage());
		return AjaxJson.getPageData(so.getDataCount(), list);
	}
	
	
	
	/** 改 - 状态(0=禁用,1=启用) */  
	@RequestMapping("updateStatus")
	@SaCheckPermission(TbPortNews.PERMISSION_CODE_EDIT)
	public AjaxJson updateStatus( Long id, String value){
		int line = SP.publicMapper.updateColumnById(TbPortNews.TABLE_NAME, "status", value, id);
		return AjaxJson.getByLine(line);
	}

	/**
	*修改口岸咨询发布状态
	*
	* @author loovi
	* @date
	*/
	@RequestMapping("updateIsRelease")
	@SaCheckPermission(TbPortNews.PERMISSION_CODE_EDIT)
	public AjaxJson updateIsRelease(Long id, int value){
		int line = tbPortNewsService.updateRelease(id, value);
		return AjaxJson.getByLine(line);
	}
	
	

	
	
	

}
