package com.pj.tb_orders;

import java.util.List;
import com.pj.utils.so.SoMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.project4sp.SP;

import cn.dev33.satoken.annotation.SaCheckPermission;


/**
 * Controller: tb_orders -- 订单表
 * @author qzy
 */
@RestController
@RequestMapping("/TbOrders/")
public class TbOrdersController {

	/** 底层 Service 对象 */
	@Autowired
	TbOrdersService tbOrdersService;

	/** 增 */
	@RequestMapping("add")
		@SaCheckPermission(TbOrders.PERMISSION_CODE_ADD)
	public AjaxJson add(TbOrders t){
		tbOrdersService.add(t);
		t = tbOrdersService.getById(SP.publicMapper.getPrimarykey());
		return AjaxJson.getSuccessData(t);
	}

	/** 删 */
	@RequestMapping("delete")
		@SaCheckPermission(TbOrders.PERMISSION_CODE_DEL)
	public AjaxJson delete(Long id){
		 tbOrdersService.delete(id);
		return AjaxJson.getSuccess();
	}

	/** 删 - 根据id列表 */
	@RequestMapping("deleteByIds")
		@SaCheckPermission(TbOrders.PERMISSION_CODE_DEL)
	public AjaxJson deleteByIds(){
		List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class);
		int line = SP.publicMapper.deleteByIds(TbOrders.TABLE_NAME, ids);
		return AjaxJson.getByLine(line);
	}

	/** 改 */
	@RequestMapping("update")
		@SaCheckPermission(TbOrders.PERMISSION_CODE_EDIT)
	public AjaxJson update(TbOrders t){
		tbOrdersService.update(t);
		return AjaxJson.getSuccess();
	}

	/** 查 - 根据id */
	@RequestMapping("getById")
		@SaCheckPermission(TbOrders.PERMISSION_CODE)
	public AjaxJson getById(Long id){
		TbOrders t = tbOrdersService.getById(id);
		return AjaxJson.getSuccessData(t);
	}

	/** 查集合 - 根据条件（参数为空时代表忽略指定条件） */
	@RequestMapping("getList")
		@SaCheckPermission(TbOrders.PERMISSION_CODE)
	public AjaxJson getList(@RequestParam(value = "ordersStatus",required = false)Integer ordersStatus) {
		SoMap so = SoMap.getRequestSoMap();
		List<TbOrders> list = tbOrdersService.getPCList(so.startPage(),ordersStatus);
		return AjaxJson.getPageData(so.getDataCount(), list);
	}



	/** 改 - 订单完成状态 (0=未完成,1=已完成) */
	@RequestMapping("updateOrderFinish")
		@SaCheckPermission(TbOrders.PERMISSION_CODE_EDIT)
	public AjaxJson updateOrderFinish(Long id, Integer value){
		int line = SP.publicMapper.updateColumnById(TbOrders.TABLE_NAME, "order_finish", value, id);
		return AjaxJson.getByLine(line);
	}




}
